from grazyna.irc.message_controller import MessageController, ping_pong
from unittest.mock import patch
from freezegun import freeze_time
from io import StringIO

import pytest
import asynctest


def test_init_without_data(protocol):
    ctrl = MessageController(protocol, [])
    assert ctrl.data == []


def test_init_with_prefix(protocol):
    data = [':Alex', 'PRIVMSG', 'Wiz', 'Hello World!']
    ctrl = MessageController(protocol, data)
    assert ctrl.data == data
    assert ctrl.command == 'PRIVMSG'


def test_init_without_prefix(protocol):
    data = ['PING', 'foo']
    ctrl = MessageController(protocol, data)
    assert ctrl.data == data
    assert ctrl.command == 'PING'


def test_execute_message_on_unknown_command(protocol):
    data = ['ROTFL']
    ctrl = MessageController(protocol, data)
    ctrl.execute_message()
    assert protocol.messages == []


def test_execute_message_without_data(protocol):
    data = []
    ctrl = MessageController(protocol, data)
    ctrl.execute_message()
    assert protocol.messages == []


@patch('builtins.open')
def test_log(open_mock, protocol):
    stream = open_mock.return_value = StringIO()
    protocol.config['main']['dir_log'] = '/foo'
    ctrl = MessageController(protocol, [])
    with freeze_time('2017-06-06 06:06:06'):
        ctrl.log('#czarnobyl', 'foobar msg')
        ctrl.log('#czarnobyl', 'another msg')
    open_mock.assert_called_once_with('/foo/#czarnobyl', 'at')
    assert stream.getvalue() == (
        '[06-06-2017 06:06:06] foobar msg\r\n'
        '[06-06-2017 06:06:06] another msg\r\n'
    )


@pytest.mark.asyncio
@asynctest.mock.patch('grazyna.irc.message_controller.asyncio')
def test_ping_pong(asyncio_mock, protocol):
    yield from ping_pong(protocol)
    asyncio_mock.sleep.assert_called_with(64)
    assert asyncio_mock.sleep.call_count == 5
    assert protocol.ping_counter == 5
    protocol.transport.close.assert_called_once_with()



'''
Created on 01-04-2013

@author: firemark
'''
from grazyna.modules import get_args_from_text, check_type
from grazyna.utils.types import range_int

import pytest


def func_test(
        bot, arg1, arg2,
        karg: range_int(0, 10)=0,
        akarg: int=2):
    pass


func_test.varkw = None
func_test.varargs = None
func_test.cmd = 'cmd'


@pytest.mark.parametrize('text,cor_args,cor_kwargs', [
    ("firemark tiramo", ["firemark", "tiramo"], {}),
    ("tiramo firemark with spaces", ["tiramo", "firemark with spaces"], {}),
    (
        "tiramo firemark with spaces karg= 2 akarg= 5",
        ["tiramo", "firemark with spaces"],
        {"karg": "2", "akarg": "5"}
    ),
    (
        '"tiramo with spaces" firemark karg=" spaces "',
        ["tiramo with spaces", "firemark"],
        {"karg": " spaces "},
    ),
    (
        'karg= firemark arg2= 20 10 ',
        ["10"],
        {"karg": "firemark", "arg2": "20"}
    ),
])
def test__get_args(text, cor_args, cor_kwargs):
    max_args = 2
    args, kwargs = get_args_from_text(text, max_args)
    assert args == cor_args
    assert kwargs == cor_kwargs


@pytest.mark.parametrize('args,kwargs,result', [
    # without kwargs
    (["t1", "t2"], {}, {"arg1": "t1", "arg2": "t2", "karg": 0, "akarg": 2}),
    # with optionals
    (
        ["t", "t", "-1", "-1"], {},
        {"arg1": "t", "arg2": "t", "karg": 0, "akarg": -1}
    ),
    (  # mixing
        ["ta", "t"], {"karg": "5", "akarg": "-1"},
        {"arg1": "ta", "arg2": "t", "karg": 5, "akarg": -1}
    ),
    (  # another
        ["t", "t"], {"karg": "15", "akarg": "10"},
        {"arg1": "t", "arg2": "t", "karg": 10, "akarg": 10}
    ),
    (  # without args
        [], {"arg1": "a", "arg2": "c", "karg": "5", "akarg": "10"},
        {"arg1": "a", "arg2": "c", "karg": 5, "akarg": 10}
    ),
])
def test__check_type(args, kwargs, result):
    assert check_type(args, kwargs, func_test) == result



from grazyna.tests.commands.base import execute_message
from grazyna.test_mocks.sender import Message


def test_kick_bot(protocol):
    protocol.config['main']['nick'] = 'firemark'
    data = [':socek!a@b', 'KICK', '#czarnobyl', 'firemark']
    ctrl = execute_message(protocol, data)
    assert protocol.messages == [Message('JOIN', '#czarnobyl')]
    ctrl.log.assert_called_once_with('#czarnobyl', 'socek KICK firemark')


def test_kick_another_person(protocol):
    protocol.config['main']['nick'] = 'firemark'
    data = [':socek!a@b', 'KICK', '#czarnobyl', 'dimmur']
    ctrl = execute_message(protocol, data)
    assert protocol.messages == []
    ctrl.log.assert_called_once_with('#czarnobyl', 'socek KICK dimmur')



from unittest.mock import Mock

from grazyna.utils import register, init_plugin
from grazyna.utils.event_loop import loop



mock_init_plugin = Mock()


@register(cmd='dummy')
def dummy(bot, a: str, b: int):
    pass


@register(reg=r'dum+y (a) (?P<b>b)')
def dummy_reg():
    pass


@register(cmd='dummier', private=True)
def dummy_private():
    pass


@loop('time', 0)
def dummy_loop():
    pass


inited_plugin = init_plugin(Mock())



from unittest.mock import Mock
from asyncio import coroutine


class AsyncMock(Mock):

    @coroutine
    def __call__(self, *args, **kwargs):
        return super().__call__(*args, **kwargs)


class Importer(object):
    protocol = None
    is_loaded = False
    is_canceled = True
    whois_heap = {}

    def __init__(self, protocol):
        self.protocol = protocol
        self.execute = AsyncMock()
        self.load = Mock()
        self.reload = Mock()
        self.remove = Mock()

    def load_all(self):
        self.is_loaded = True

    def cancel_tasks(self):
        self.is_canceled = True



from grazyna.tests.commands.base import execute_message
from grazyna.irc.exc import NoSuchNickError
from grazyna.irc.models import WhoisFuture


def test_command_no_such_nick(protocol):
    data = [
        ':server', '401', 'bot',
        'socek', ':No such nick/channel',
    ]
    future = protocol.whois_heap['socek'] = WhoisFuture()
    execute_message(protocol, data)
    assert protocol.whois_heap == {}
    assert future.exception() == NoSuchNickError('socek')



from grazyna.auths import FreenodeAuth
from grazyna.test_mocks.sender import SayMessage


def test_freenode_auth(protocol):
    auth = FreenodeAuth('klocek')
    auth.auth(protocol)
    assert protocol.messages == [
        SayMessage('nickserv', 'identify klocek'),
    ]



from configparser import ConfigParser
from importlib import import_module
import re


class MyConfigParser(ConfigParser):

    def getlist(self, section, key, seperator=','):
        return re.split(' *%s *' % seperator, self[section][key])

    def getmodule(self, section, key):
        module_fullpath = self[section][key]
        module_path, _, cls_path = module_fullpath.rpartition('.') 
        module = import_module(module_path) 
        return getattr(module, cls_path)


def create_empty_config():
    return MyConfigParser(
        comment_prefixes=[';', '//'],
        dict_type=dict,
    )


def create_config(stream):
    config = create_empty_config()
    config.read_file(stream)

    return config



from grazyna.modules import Plugin


def foo():
    pass


def test_plugin_init():
    plugin = Plugin('foobar', 'grazyna.foo', [foo])
    assert plugin.temp == {}
    assert plugin.name == 'foobar'
    assert plugin.module_path == 'grazyna.foo'
    assert plugin.config == {}
    assert plugin == [foo]


def test_plugin_repr():
    plugin = Plugin('foobar', 'grazyna.foo', [])
    assert repr(plugin) == "Plugin('foobar')"



from unittest.mock import Mock, MagicMock
from asynctest.mock import CoroutineMock, patch as async_patch
from freezegun.api import freeze_time

from grazyna.irc.models import User
from grazyna.models import Message
from grazyna.modules import ModuleManager, Plugin, ExecutedCounter
from grazyna.request import RequestBot
from grazyna.test_mocks.sender import SayMessage
from grazyna.test_mocks.dummy_plugin import dummy

import pytest


@pytest.fixture
def manager(protocol):
    manager = ModuleManager(protocol)
    manager.config.add_section('plugins')
    manager.config.set('plugins', 'dummy', 'grazyna.test_mocks.dummy_plugin')
    return manager


@pytest.fixture
def manager_with_db(protocol_with_db):
    manager = ModuleManager(protocol_with_db)
    manager.config.add_section('plugins')
    manager.config.set('plugins', 'dummy', 'grazyna.test_mocks.dummy_plugin')
    return manager


@pytest.mark.asyncio
def test_load_all(manager: ModuleManager):
    yield from []  # noqa
    manager.load_all()

    plugin = manager.plugins['dummy']
    assert plugin.name == 'dummy'
    assert plugin.module_path == 'grazyna.test_mocks.dummy_plugin'
    assert plugin.config == {}

    from grazyna.test_mocks.dummy_plugin import (
        dummy, dummy_private, dummy_reg, inited_plugin
    )
    assert set(plugin) == {dummy, dummy_private, dummy_reg}
    inited_plugin.assert_called_once_with(plugin, manager.protocol)


@pytest.mark.asyncio
def test_load_all_with_config(manager: ModuleManager):
    yield from []  # noqa
    manager.config.add_section('plugin:dummy')
    manager.config.set('plugin:dummy', 'foo', 'bar')
    manager.load_all()

    plugin = manager.plugins['dummy']
    assert plugin.config == {'foo': 'bar'}


def test_cancel_tasks(manager: ModuleManager):
    manager.plugins['dummy'] = Plugin('a', 'b', [])
    manager.cancel_tasks()
    assert manager.plugins['dummy'].future.cancelled


def test_reload(manager: ModuleManager):
    manager.plugins['dummy'] = Plugin('a', 'path', [])
    manager.load = Mock()
    manager.reload('dummy')
    assert manager.plugins['dummy'].future.cancelled
    manager.load.called_once_with('dummy', 'path')


def test_remove(manager: ModuleManager):
    plugin = manager.plugins['dummy'] = Plugin('a', 'path', [])
    manager.load = Mock()
    manager.remove('dummy')
    assert plugin.future.cancelled
    assert manager.plugins == {}


@pytest.mark.asyncio
def test_execute_as_cmd(manager: ModuleManager):
    manager.execute_command = CoroutineMock()
    yield from manager.execute('#czarnobyl', User('a!b@c'), '.dummy lol 42')
    manager.execute_command.assert_called_once_with(
        'dummy', 'lol 42', False, '#czarnobyl', User('a!b@c'),
    )


@pytest.mark.asyncio
def test_execute_as_cmd_and_reply(manager: ModuleManager):
    manager.execute_command = CoroutineMock()
    yield from manager.execute('#czarnobyl', User('a!b@c'), 'bot: dummy lol')
    manager.execute_command.assert_called_once_with(
        'dummy', 'lol', False, '#czarnobyl', User('a!b@c'),
    )


@pytest.mark.asyncio
def test_execute_as_cmd_and_private(manager: ModuleManager):
    manager.execute_command = CoroutineMock()
    yield from manager.execute('socek', User('a!b@c'), '.dummy lol')
    manager.execute_command.assert_called_once_with(
        'dummy', 'lol', True, 'socek', User('a!b@c'),
    )


@pytest.mark.asyncio
def test_execute_as_reg(manager: ModuleManager):
    manager.execute_regs = CoroutineMock()
    yield from manager.execute('#czarnobyl', User('a!b@c'), 'dummmmmmy')
    manager.execute_regs.assert_called_once_with(
        'dummmmmmy', False, '#czarnobyl', User('a!b@c'),
    )


@pytest.mark.asyncio
def test_execute_command(manager: ModuleManager):
    manager.execute_func = CoroutineMock()
    manager.find_message_in_db = CoroutineMock()
    manager.load_all()
    yield from manager.execute_command(
        cmd='dummy',
        text='lol b=42',
        private=False,
        channel='#czarnobyl',
        user=User('a!b@c'),
    )

    assert not manager.find_message_in_db.called

    from grazyna.test_mocks.dummy_plugin import dummy
    manager.execute_func.assert_called_once_with(
        dummy,
        manager.plugins['dummy'],
        False,
        '#czarnobyl',
        User('a!b@c'),
        ['lol'],
        {'b': '42'},
    )


@pytest.mark.asyncio
def test_execute_not_exists_command(manager: ModuleManager):
    manager.execute_func = CoroutineMock()
    manager.find_message_in_db = CoroutineMock()
    manager.load_all()
    yield from manager.execute_command(
        cmd='szalona-dziewczyna',
        text='lol b=42',
        private=False,
        channel='#czarnobyl',
        user=User('a!b@c'),
    )

    assert not manager.execute_func.called

    manager.find_message_in_db.assert_called_once_with(
        'szalona-dziewczyna',
        '#czarnobyl',
        'lol b=42',
    )


@pytest.mark.asyncio
def test_execute_regs(manager: ModuleManager):
    manager.execute_func = CoroutineMock()
    manager.load_all()
    yield from manager.execute_regs(
        msg='dummmmy a b',
        private=False,
        channel='#czarnobyl',
        user=User('a!b@c'),
    )

    from grazyna.test_mocks.dummy_plugin import dummy_reg
    manager.execute_func.assert_called_once_with(
        dummy_reg,
        manager.plugins['dummy'],
        False,
        '#czarnobyl',
        User('a!b@c'),
        ('a', 'b'),
        {},
    )


@pytest.mark.asyncio
@async_patch('grazyna.modules.RequestBot')
def test_execute_func(bot_cls, manager: ModuleManager):
    manager.load_all()
    mock = MagicMock(spec=dummy, name='dummy')
    mock.__wrapped__ = dummy.__wrapped__
    bot = bot_cls.return_value = Mock(spec=RequestBot, name='bot')

    yield from manager.execute_func(
        func=mock,
        plugin=manager.plugins['dummy'],
        private=False,
        channel='#czarnobyl',
        user=User('a!b@c'),
        args=['lol'],
        kwargs={'b': '42'},
    )

    bot_cls.assert_called_once_with(
        protocol=manager.protocol,
        private=False,
        chan='#czarnobyl',
        config={'__nick__': 'bot'},
        plugin=manager.plugins['dummy'],
        user=User('a!b@c'),
        temp={},
    )

    mock.assert_called_once_with(bot, a='lol', b=42)


@pytest.mark.asyncio
def test_find_message_in_db(manager_with_db: ModuleManager):
    manager_with_db.load_all()
    with manager_with_db.protocol.get_session() as session:
        msg = Message()
        msg.channel = '#czarnobyl'
        msg.key = 'gjm'
        msg.message = '$0: $1gram juz miesiac; $@'
        session.add(msg)

    yield from manager_with_db.find_message_in_db(
        cmd='gjm',
        channel='#czarnobyl',
        text='lol kilo',
    )

    assert manager_with_db.protocol.messages == [
        SayMessage(
            '#czarnobyl',
            'lol: kilogram juz miesiac; lol kilo',
        )
    ]


def test_is_blocked(manager: ModuleManager):
    user = User('a!0@c')
    another_user = User('b!1@c')
    counters = manager.executed_counters
    with freeze_time('2017-01-01 12:00:00'):
        counter = ExecutedCounter()
        counter.counter = 50
        counters[user.prefix] = counter

    with freeze_time('2017-01-01 10:00:00'):
        counter = ExecutedCounter()
        counter.counter = 50
        counters[another_user.prefix] = counter

    with freeze_time('2017-01-01 12:00:00'):
        assert manager.is_blocked(user) is True
        assert manager.is_blocked(another_user) is False

    counters = manager.executed_counters
    assert counters[user.prefix].counter == 50
    assert counters[another_user.prefix].counter == 1



from grazyna.db import get_session
from grazyna.irc.sender import IrcSender as DefaultIrcSender
from grazyna.config import create_empty_config
from grazyna.test_mocks.importer import Importer

from unittest.mock import Mock
from asyncio import BaseTransport



class IrcSender(DefaultIrcSender):
    ready = False
    messages = None
    config = None
    db = None
    transport = None
    ping_counter = 0

    def __init__(self):
        super().__init__()
        self.messages = []
        self.config = config = create_empty_config()
        self.transport = Mock(spec=BaseTransport)
        self.importer = Importer(self)
        config.read_dict(dict(
            main={
                'nick': 'bot',
                'importer': 'grazyna.test_mocks.importer.Importer',
                'command-prefix': '.',
                'time_to_block': '30',
                'executed_commands_per_time': '20',
                'db_uri': 'sqlite://',
            },
            auth=dict(
                module='grazyna.auths.NonAuth',
            )
        ))

    def send(self, *args):
        self.messages.append(Message(*args))

    def get_session(self):
        return get_session(self.db).scope()


class Message(list):

    def __init__(self, *args):
        arg, *args = args
        super().__init__([arg.upper()] + args)


class SayMessage(Message):

    def __init__(self, to, msg):
        super().__init__('PRIVMSG', to, ':' + msg)



from datetime import datetime

from grazyna.utils import register

@register(cmd='weekend')
def weekend(bot):
    """
    Answer to timeless question - are we at .weekend, yet?
    """
    current_date = datetime.now()
    day = current_date.weekday()
    nick = bot.user.nick
    if day in (5, 6):
        answer = "Oczywiście %s - jest weekend. Omawiamy tylko lajtowe tematy, ok?" % nick
    else:
        str_day = datetime.strftime(current_date, "%A")
        answer = "%s - dopiero %s, musisz jeszcze poczekać..." % (nick, str_day)

    bot.reply(answer)



# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4

import asyncio
import re
import string
from html.parser import HTMLParser, unescape
from io import BytesIO
from random import choice

from aiohttp import ClientError, ClientSession, TCPConnector

from ..utils import register


default_charset = 'utf-8'

TIMEOUT = 2
MAX_CHUNK_BYTES = 1024
MAX_CHUNKS = 8
re_charsets = (
    re.compile(rb"< *meta.+?charset=([\"']?)([\w\s\_-]+)\1[^>]*>"),
    re.compile(
        rb"< *meta.+?content=([\"']?)[^;]*; +charset=([\w\s\_-]+)\1[^>]*>"),
)
re_space = re.compile(r'\s+')


class TitleParser(HTMLParser):
    title_tag = False
    first = False
    title = None
    charset = "utf-8"

    def handle_starttag(self, tag, attrs):
        if self.first:
            return
        if tag == "title":
            self.title_tag = True
            self.first = True

    def handle_data(self, data):
        if self.title_tag:
            self.title = (self.title or '') + data

    def handle_endtag(self, tag):
        if tag == "title":
            self.title_tag = False

    @classmethod
    def get_title(cls, data):
        parser = cls(convert_charrefs=True)
        try:
            parser.feed(data)
        except:  # many bugs lol
            return None
        title = parser.title
        if title is None:
            return None
        title = title.strip()
        if title:
            return re_space.sub(' ', unescape(title))


@asyncio.coroutine
def get_response(address, headers=None, redirect=True,
                 ssl=False, session=None):
    """
    :return: tuple of message and response
    :rtype: (string, aiohttp.Request)
    """
    try:
        if session is None:
            session = ClientSession(
                connector=TCPConnector(
                    verify_ssl=ssl,
                    conn_timeout=TIMEOUT
                )
            )
        resp = yield from session.get(
            address,
            headers=headers,
            allow_redirects=redirect
        )
        try:
            if resp.status != 200:
                return None, None
            raw_io = BytesIO()
            for _ in range(MAX_CHUNKS):
                chunk = yield from resp.content.read(MAX_CHUNK_BYTES)
                if not chunk:
                    break
                raw_io.write(chunk)
            raw = raw_io.getvalue()
        finally:
            resp.close()
    except ClientError:
        return None, None
    finally:
        session.close()

    charset = default_charset
    for re_charset in re_charsets:
        charset_match = re_charset.search(raw)
        if charset_match:
            charset = charset_match.group(2).decode()
            break

    try:
        msg = raw.decode(charset, errors='replace')
    except LookupError:
        return None, None
    return msg, resp


def address_in_blacklist(address, blacklist):
    for entry in blacklist:
        if address.startswith(entry):
            return True


@asyncio.coroutine
def check_title(bot, address, ssl):
    if address_in_blacklist(address, bot.config.values()):
        return

    msg, resp = yield from get_response(address, ssl=ssl)

    if resp is None:
        return

    headers = resp.headers
    if "content-type" not in headers:
        return
    cont_type = headers["content-type"]
    if not cont_type.startswith("text/html"):
        return
    if not msg:
        return

    title = TitleParser.get_title(msg)

    if not title:
        return
    if title.startswith('Rick Astley'):
        title = 'Funny %s - YouTube' % choice((
            'cats', 'dogs', 'firemark'
        ))
    bot.say("⚡ %s" % title)


@register(reg=r'http(s?)://(\S+)|(www\.\S+)')
def title(bot, ssl: lambda s: s == 's', address, address_another):
    address = address or address_another

    # ensure that characters like \x01 are removed
    address = "".join([char for char in address if char in string.printable])
    if not address.startswith('http'):
        address = "http://" + address

    yield from check_title(bot, address, ssl)



from grazyna.tests.commands.base import execute_message


def test_part_without_reason(protocol):
    ctrl = execute_message(protocol, [':socek!a@b', 'PART', '#czarnobyl'])
    assert protocol.messages == []
    ctrl.log.assert_called_once_with('#czarnobyl', 'socek::PART')


def test_part_with_reason(protocol):
    data = [':socek!a@b', 'PART', '#czarnobyl', 'dunno lol']
    ctrl = execute_message(protocol, data)
    assert protocol.messages == []
    ctrl.log.assert_called_once_with('#czarnobyl', 'socek::PART[dunno lol]')



from asyncio import Future
from grazyna.irc.client import IrcClient as DefaultIrcClient
from grazyna.irc.models import WhoisData
from grazyna.test_mocks.sender import IrcSender, Message
from grazyna.config import create_empty_config
from unittest.mock import Mock, patch, call

import pytest
import asyncio


class IrcClient(DefaultIrcClient, IrcSender):
    pass


def make_client(**dict_cfg):
    config = create_empty_config()
    config.read_dict({'main': {
        'importer': 'grazyna.test_mocks.importer.Importer',
    }})
    config.read_dict(dict_cfg)
    return IrcClient(config, Future())


def test_init():
    client = make_client()
    assert client.importer.is_loaded is True
    assert client.importer.protocol is client


def test_connection_made():
    client = make_client(main=dict(
        nick='socek',
        password='klocek',
        ircname='superSocek',
        realname='realSocek',
    ))
    transport = Mock()
    client.connection_made(transport)
    assert client.transport is transport
    assert client.messages == [
        Message('PASS', 'klocek'),
        Message('NICK', 'socek'),
        Message('USER', 'superSocek', 'wr', '*', ':realSocek'),
    ]


@patch('grazyna.irc.client.MessageController')
def test_data_received(message_controller):
    client = make_client(main=dict(codecs=['utf-8']))
    msg1 = b':Angel PRIVMSG Wiz :Hello tl;dr\r\n'
    msg2 = b'PONG csd.bu.edu tolsun.oulu.fi\r\n'
    client.data_received(msg1 + msg2)
    assert message_controller.call_args_list == [
        call(client, [':Angel', 'PRIVMSG', 'Wiz', 'Hello tl;dr']),
        call(client, ['PONG', 'csd.bu.edu', 'tolsun.oulu.fi']),
    ]


@patch('grazyna.irc.client.MessageController')
def test_wrong_data_received(message_controller):
    client = make_client(main=dict(codecs=['utf-8']))
    client.data_received(b'\xFF\r\n')
    assert message_controller.call_args_list == []


def test_connection_lost():
    client = make_client()
    exc = Exception('socek')
    client.connection_lost(exc)
    assert client.connection_lost_future.exception() == exc
    assert client.importer.is_canceled is True


def test_db_scope():
    client = make_client(main=dict(db_uri='sqlite://'))
    session = client.get_session()
    assert session is not None


@pytest.mark.asyncio
def test_whois():
    client = make_client()

    @asyncio.coroutine
    def set_whois():
        yield from asyncio.sleep(0.5)
        future = client.whois_heap['socek']
        future.data.nick = 'socek'
        future.set_result(future.data)

    result, _ = yield from asyncio.gather(
        client.whois('socek'),
        set_whois()
    )

    assert isinstance(result, WhoisData)
    assert result.nick == 'socek'



"""
to setup this please visit
https://developers.google.com/custom-search/json-api/v1/using_rest
"""
from grazyna.utils import register
from grazyna import format

from aiohttp import request

GOOGLE_URL = 'https://www.googleapis.com/customsearch/v1'


@register(cmd='g')
def google(bot, query):
    """get first result of google query"""

    params = dict(
        key=bot.config['key'],
        cx=bot.config['cx'],
        prettyPrint=False,
        q=query,
    )
    response = yield from request('GET', GOOGLE_URL, params=params)
    json_data = yield from response.json()
    error = json_data.get('error')
    if error:
        bot.say('err: %s' % error['message'])
    items = json_data.get('items', [])

    try:
        result = items[0]
    except IndexError:
        bot.reply('not found')
        return

    title = result['title']
    url = result['link']
    bot.say('{}: {}'.format(format.bold(title[:50]), url))



from grazyna.tests.commands.base import execute_message
from grazyna.irc.models import User

import pytest


@pytest.mark.asyncio
def test_notice(protocol_with_importer, event_loop):
    yield from []  # noqa - turn on global event_loop
    data = [':socek!a@b', 'NOTICE', '#czarnobyl', 'message']
    ctrl = execute_message(protocol_with_importer, data)

    def check(ctrl):  # wait for another coroutines
        importer = protocol_with_importer.importer
        assert protocol_with_importer.messages == []
        assert importer.execute.assert_called_once_with(
            '#czarnobyl', User('socek!a@b'), 'message'
        )

        ctrl.log.assert_called_once_with('#czarnobyl', '<socek> message')
    event_loop.call_soon(check, ctrl)



from grazyna.tests.commands.base import execute_message
from grazyna.irc.models import WhoisFuture


def test_whois_user(protocol):
    data = [
        ':server', '311', 'bot',
        'socek', 'realsocek', 'onet.pl', '*', 'the socketor',
    ]
    future = protocol.whois_heap['socek'] = WhoisFuture()
    execute_message(protocol, data)
    assert future.data.nick == 'socek'
    assert future.data.ircname == 'the socketor'
    assert future.data.host == 'onet.pl'
    assert future.data.realname == 'realsocek'


def test_whois_user_without_trigger(protocol):
    data = [
        ':server', '311', 'bot',
        'socek', 'realsocek', 'onet.pl', '*', 'the socketor',
    ]
    execute_message(protocol, data)
    assert 'socek' not in protocol.whois_heap


def test_whois_server(protocol):
    data = [':server', '312', 'bot', 'socek', 'banana server']
    future = protocol.whois_heap['socek'] = WhoisFuture()
    execute_message(protocol, data)
    assert future.data.server == 'banana server'


def test_whois_idle(protocol):
    data = [':server', '317', 'bot', 'socek', '30', 'seconds idle']
    future = protocol.whois_heap['socek'] = WhoisFuture()
    execute_message(protocol, data)
    assert future.data.idle == 30


def test_whois_channels(protocol):
    data = [':server', '319', 'bot', 'socek', '#foo #bar']
    future = protocol.whois_heap['socek'] = WhoisFuture()
    execute_message(protocol, data)
    assert future.data.channels == ['#foo', '#bar']


def test_whois_account(protocol):
    data = [':server', '330', 'bot', 'socek', 'socketor']
    future = protocol.whois_heap['socek'] = WhoisFuture()
    execute_message(protocol, data)
    assert future.data.account == 'socketor'


def test_whois_end(protocol):
    data = [':server', '318', 'bot', 'socek', 'End of /WHOIS list']
    future = protocol.whois_heap['socek'] = WhoisFuture()
    execute_message(protocol, data)
    assert future.result() is future.data



from grazyna.tests.commands.base import execute_message
from grazyna.test_mocks.sender import Message

import pytest

data = [':irc.server.net', '005', 'bot', 'CHANTYPES=#', 'NICKLEN=30']


def test_start_if_is_started(protocol):
    protocol.ready = True
    execute_message(protocol, data)
    assert protocol.messages == []


@pytest.mark.asyncio
def test_start_if_is_not_started(protocol):
    yield from []  # noqa - turn on global event_loop
    protocol.ready = False
    protocol.config['main']['channels'] = '#foo, #baar'
    execute_message(protocol, data)
    assert protocol.ready is True
    assert protocol.messages == [
        Message('JOIN', '#foo'),
        Message('JOIN', '#baar'),
    ]



from grazyna.request import RequestBot
from grazyna.test_mocks.sender import SayMessage, Message
from grazyna.irc.models import User


def make_bot(protocol, **kwargs):
    return RequestBot(protocol, user=User('socek!a@b'), chan='#czarnobyl', **kwargs)


def test_init(protocol):
    bot = RequestBot(protocol)
    assert bot.protocol is protocol


def test_nick_chan_on_privmsg(protocol):
    bot = make_bot(protocol, private=True)
    assert bot.nick_chan == 'socek'


def test_nick_chan_on_channel(protocol):
    bot = make_bot(protocol)
    assert bot.nick_chan == '#czarnobyl'


def test_say(protocol):
    bot = make_bot(protocol)
    bot.say('foobar')
    assert protocol.messages == [SayMessage('#czarnobyl', 'foobar')]


def test_notice(protocol):
    bot = make_bot(protocol)
    bot.notice('foobar')
    assert protocol.messages == [Message('NOTICE', 'socek', ':foobar')]


def test_reply(protocol):
    bot = make_bot(protocol)
    bot.reply('foobar')
    assert protocol.messages == [SayMessage('#czarnobyl', 'socek: foobar')]


def test_kick(protocol):
    bot = make_bot(protocol)
    bot.kick(who='firemark', why='dunno lol')
    assert protocol.messages == [Message('KICK', '#czarnobyl', 'firemark', ':dunno lol')]


def test_kick_on_private(protocol):
    bot = make_bot(protocol, private=True)
    bot.kick()  # is not possible to kick on private message
    assert protocol.messages == []
    

def test_private_say(protocol):
    bot = make_bot(protocol)
    bot.private_say('foobar')
    assert protocol.messages == [SayMessage('socek', 'foobar')]


def test_command(protocol):
    bot = make_bot(protocol)
    bot.command('WTF')
    assert protocol.messages == [Message('WTF')]


def test_command_msg(protocol):
    bot = make_bot(protocol)
    bot.command_msg('WTF', 'dunno lol')
    assert protocol.messages == [Message('WTF', ':dunno lol')]


def test_mode(protocol):
    bot = make_bot(protocol)
    bot.mode('#czarnobyl', '+oo', 'lol')
    assert protocol.messages == [Message('MODE', 'lol', '#czarnobyl', '+oo')]



from grazyna.utils.types import range_int, is_chan, regexp

import pytest


def test_range_int():
    typer = range_int(1, 3)
    assert typer('4') == 3
    assert typer('0') == 1
    assert typer('2') == 2


def test_range_int_without_ranges():
    typer = range_int()
    assert typer('0') == 0


def test_is_chan():
    assert is_chan('#czarnobyl') == '#czarnobyl'
    with pytest.raises(TypeError):
        is_chan('socek')


def test_regexp():
    typer = regexp(r'001+')
    match = typer('0011')
    assert match.group(0) == '0011'
    with pytest.raises(TypeError):
        typer('2')




from grazyna import format


def test__bold():
    assert format.bold('socek') == '\x02socek\x02'


def test__underline():
    assert format.underline('socek') == '\x1Fsocek\x1F'


def test__italic():
    assert format.italic('socek') == '\x16socek\x16'


def test__color_without_background():
    msg = format.color('socek', format.color.white)
    assert msg == '\x030socek\x03'


def test__color_with_background():
    msg = format.color('socek', format.color.white, format.color.black)
    assert msg == '\x030,1socek\x03'



from grazyna.modules import ExecutedCounter
from freezegun import freeze_time
from datetime import datetime


def test_counter_init():
    with freeze_time('2016-06-06'):
        counter = ExecutedCounter()
    assert counter.counter == 0
    assert counter.last_time == datetime(2016, 6, 6)


def test_counter_increase():
    counter = ExecutedCounter()
    counter.inc()
    assert counter.counter == 1



from grazyna.auths import AbstractAuth
import pytest


@pytest.fixture
def auth():
    return AbstractAuth()


def test_abstract_auth_add_permission(auth):
    with pytest.raises(NotImplementedError):
        auth.add_permission('test', '#test', 0)


def test_abstract_auth_check_permission(auth):
    with pytest.raises(NotImplementedError):
        auth.check_permission('test', '#test', 0)


def test_abstract_auth__auth_method(auth):
    with pytest.raises(NotImplementedError):
        auth.auth()



from grazyna.modules import ModuleManager
from grazyna.irc.models import User, WhoisData
from unittest.mock import Mock

import asyncio
import pytest


def create_func(cmd='test'):
    def func_test(bot, xx):
        pass
    func_test.varkw = None
    func_test.varargs = None
    func_test.cmd = cmd
    return func_test


def cmd_is_good(
        cfg, func, cmd=None, private=False,
        channel=None, user=None, whois=None):
    protocol = Mock(name='protocol')

    @asyncio.coroutine
    def whois_func(nick):
        return whois
    protocol.whois = whois_func
    plugin = Mock(name='plugin')
    plugin.name = 'test'
    mm = ModuleManager(protocol)
    mm.get_plugin_cfg = Mock(name='get_plugin_cfg', return_value=cfg)
    return mm.cmd_is_good(plugin, func, cmd, private, channel, user)


@pytest.mark.asyncio
def test__wrong_cmd():
    assert not (yield from cmd_is_good(
        cfg={},
        func=create_func(cmd='wrong_cmd'),
        cmd='correct_cmd',
    ))


@pytest.mark.asyncio
def test__wrong_formatted_cmd():
    assert not (yield from cmd_is_good(
        cfg={'foo': 'bar'},
        func=create_func(cmd='cmd_{foo}'),
        cmd='cmd_foo',
    ))


@pytest.mark.asyncio
def test__cmd_is_private():
    assert (yield from cmd_is_good(
        cfg={'foo': 'bar'},
        func=create_func(cmd='cmd_{foo}'),
        cmd='cmd_bar',
        private=True,
        channel=None,
    ))


@pytest.mark.asyncio
def test__channel_in_whitelist():
    assert (yield from cmd_is_good(
        cfg={'whitelist': '#bar,#foo'},
        func=create_func(cmd='cmd'),
        cmd='cmd',
        channel='#bar',
    ))


@pytest.mark.asyncio
def test__channel_not_in_whitelist():
    assert not (yield from cmd_is_good(
        cfg={'whitelist': '#bar,#foo'},
        func=create_func(cmd='cmd'),
        cmd='cmd',
        channel='#barfoo',
    ))


@pytest.mark.asyncio
def test__channel_in_blacklist():
    assert not (yield from cmd_is_good(
        cfg={'blacklist': '#bar,#foo'},
        func=create_func(cmd='cmd'),
        cmd='cmd',
        channel='#foo',
    ))


@pytest.mark.asyncio
def test__channel_not_in_blacklist():
    assert (yield from cmd_is_good(
        cfg={'blacklist': '#bar,#foo'},
        func=create_func(cmd='cmd'),
        cmd='cmd',
        channel='#barfoo',
    ))


@pytest.mark.asyncio
def test__user_and_channel_in_whitelist():
    whois = WhoisData()
    whois.account = 'FooNick'
    assert (yield from cmd_is_good(
        cfg={'whitelist': '#bar,#foobar{Barnick},#foo{FooNick;BarNick}'},
        func=create_func(cmd='cmd'),
        cmd='cmd',
        channel='#foo',
        user=User('foo!foo@foo'),
        whois=whois
    ))


@pytest.mark.asyncio
def test__user_in_whitelist():
    whois = WhoisData()
    whois.account = 'BarNick'
    assert (yield from cmd_is_good(
        cfg={'whitelist': '#foobar,#barfoo{FooNick},*{FooNick;BarNick}'},
        func=create_func(cmd='cmd'),
        cmd='cmd',
        channel='#bar',
        user=User('foo!foo@foo'),
        whois=whois,
    ))


@pytest.mark.asyncio
def test__user_and_channel_in_blacklist():
    whois = WhoisData()
    whois.account = 'BarNick'
    assert not (yield from cmd_is_good(
        cfg={'whitelist': '#foobar,#barfoo{BarNick},*{FooNick}'},
        func=create_func(cmd='cmd'),
        cmd='cmd',
        channel='#bar',
        user=User('foo!foo@foo'),
        whois=whois,
    ))



from grazyna.auths import QuakenetAuth
from grazyna.test_mocks.sender import SayMessage


def test_quakenet_auth(protocol):
    auth = QuakenetAuth('socek', 'klocek')
    auth.auth(protocol)
    assert protocol.messages == [
        SayMessage('Q@CServe.quakenet.org', 'AUTH socek klocek'),
    ]



from grazyna.irc.message_controller import MessageController
from unittest.mock import Mock


def execute_message(protocol, data):
    ctrl = MessageController(protocol, data)
    ctrl.log = Mock()
    ctrl.execute_message()
    return ctrl



from grazyna.tests.commands.base import execute_message
from grazyna.test_mocks.sender import Message


def test_ping(protocol):
    execute_message(protocol, ['PING', 'foobar'])
    assert protocol.messages == [Message('PONG', 'foobar')]


def test_pong(protocol):
    execute_message(protocol, ['PONG', 'foobar'])
    assert protocol.messages == []
    assert protocol.ping_counter == 0



from grazyna.tests.commands.base import execute_message


def test_join(protocol):
    ctrl = execute_message(protocol, [':socek!a@b', 'JOIN', '#czarnobyl'])
    assert protocol.messages == []
    ctrl.log.assert_called_once_with('#czarnobyl', 'socek(a@b)::JOIN')
